package br.usp.ime.blog.repositorios.impl.jdbc;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import br.usp.ime.blog.dominio.BlogRecomendado;
import br.usp.ime.blog.repositorios.Blogroll;
import br.usp.ime.blog.repositorios.RepositorioException;

public class BlogrollJdbc extends BaseRepositorioJdbc implements Blogroll {

	public void adicionaBlog(BlogRecomendado blog) throws RepositorioException {
		try {
			PreparedStatement stmt = preparaQuery("insert into blogroll (titulo, url) values (?, ?)");
			stmt.setString(1, blog.getTitulo());
			stmt.setString(2, blog.getUrl());
			stmt.execute();
		} catch (SQLException ex) {
			throw new RepositorioException(ex);
		}
	}

	public void apagaBlog(int id) throws RepositorioException {
		try {
			PreparedStatement stmt = preparaQuery("delete from blogroll where id=?");
			stmt.setInt(1, id);
			stmt.execute();
		} catch (SQLException ex) {
			throw new RepositorioException(ex);
		}
	}

	public void atualizaBlog(BlogRecomendado blog) throws RepositorioException {
		try {
			PreparedStatement stmt = preparaQuery("update blogroll set titulo=?, url=? where id=?");
			stmt.setString(1, blog.getTitulo());
			stmt.setString(2, blog.getUrl());
			stmt.setInt(3, blog.getId());
			stmt.execute();
		} catch (SQLException ex) {
			throw new RepositorioException(ex);
		}
	}

	public List<BlogRecomendado> listaBlogs() throws RepositorioException {
		try {
			ResultSet rs = executaQuery("select * from blogroll");
			List<BlogRecomendado> listaDeBlogs = new ArrayList<BlogRecomendado>();
			while (rs.next()) {
				listaDeBlogs.add(popula(rs));
			}
			return listaDeBlogs;
		} catch (SQLException ex) {
			throw new RepositorioException(ex);
		}
	}

	private BlogRecomendado popula(ResultSet rs) throws SQLException {
		BlogRecomendado blog = new BlogRecomendado();
		blog.setId(rs.getInt("id"));
		blog.setTitulo(rs.getString("titulo"));
		blog.setUrl(rs.getString("url"));
		return blog;
	}

	public BlogRecomendado pegaBlog(int id) throws RepositorioException {
		try {
			PreparedStatement stmt = preparaQuery("select * from blogroll where id = ?");
			stmt.setInt(1, id);
			ResultSet rs = stmt.executeQuery();

			if (rs.next()) {
				return popula(rs);
			}

			return null;
		} catch (SQLException ex) {
			throw new RepositorioException(ex);
		} finally {
			finaliza();
		}
	}
}