package br.usp.ime.blog.repositorios.impl.jdbc;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import br.usp.ime.blog.dominio.Pagina;
import br.usp.ime.blog.repositorios.RepositorioDePaginas;
import br.usp.ime.blog.repositorios.RepositorioException;

public class RepositorioDePaginasJdbc extends BaseRepositorioJdbc implements
		RepositorioDePaginas {

	public void adicionaPagina(Pagina pagina) throws RepositorioException {
		try {
			PreparedStatement stmt = preparaQuery("insert into paginas (titulo, conteudo) values (?, ?)");
			stmt.setString(1, pagina.getTitulo());
			stmt.setString(2, pagina.getConteudo());
			stmt.execute();
		} catch (SQLException ex) {
			throw new RepositorioException(ex);
		} finally {
			finaliza();
		}
	}

	public void apagaPagina(int id) throws RepositorioException {
		try {
			PreparedStatement stmt = preparaQuery("delete from paginas where id=?");
			stmt.setInt(1, id);
			stmt.execute();
		} catch (SQLException ex) {
			throw new RepositorioException(ex);
		} finally {
			finaliza();
		}
	}

	public void atualizaPagina(Pagina pagina) throws RepositorioException {
		try {
			PreparedStatement stmt = preparaQuery("update paginas set titulo=?, conteudo=? where id=?");
			stmt.setString(1, pagina.getTitulo());
			stmt.setString(2, pagina.getConteudo());
			stmt.setInt(3, pagina.getId());
			stmt.execute();
		} catch (SQLException ex) {
			throw new RepositorioException(ex);
		} finally {
			finaliza();
		}
	}

	public List<Pagina> listaPaginas() throws RepositorioException {
		try {
			PreparedStatement stmt = preparaQuery("select * from paginas order by titulo");
			ResultSet rs = stmt.executeQuery();

			List<Pagina> paginas = new ArrayList<Pagina>();
			while (rs.next()) {
				paginas.add(popula(rs));
			}
			return paginas;
		} catch (SQLException ex) {
			throw new RepositorioException(ex);
		} finally {
			finaliza();
		}
	}

	public Pagina pegaPagina(int id) throws RepositorioException {
		try {
			PreparedStatement stmt = preparaQuery("select * from paginas where id = ?");
			stmt.setInt(1, id);
			ResultSet rs = stmt.executeQuery();

			if (rs.next()) {
				return popula(rs);
			}
			return null;
		} catch (SQLException ex) {
			throw new RepositorioException(ex);
		} finally {
			finaliza();
		}
	}

	private Pagina popula(ResultSet rs) throws SQLException {
		Pagina pagina = new Pagina();
		pagina.setId(rs.getInt("id"));
		pagina.setTitulo(rs.getString("titulo"));
		pagina.setConteudo(rs.getString("conteudo"));
		return pagina;
	}
}
