package br.usp.ime.blog.repositorios.impl.jdbc;

import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import br.usp.ime.blog.dominio.Avaliacao;
import br.usp.ime.blog.dominio.Blogueiro;
import br.usp.ime.blog.dominio.Comentario;
import br.usp.ime.blog.dominio.Post;
import br.usp.ime.blog.repositorios.RepositorioDePosts;
import br.usp.ime.blog.repositorios.RepositorioException;

public class RepositorioDePostsJdbc extends BaseRepositorioJdbc implements
		RepositorioDePosts {

	public void adicionaPost(Post post) throws RepositorioException {
		try {
			PreparedStatement stmt = preparaQuery("insert into post "
					+ "(titulo, conteudo, publicado, data, blogueiro_fk, qtde_avaliacoes, media_avaliacoes) "
					+ "values (?, ?, ?, ?, ?, 0, 0)");

			java.sql.Date data = new java.sql.Date(post.getData().getTime()
					.getTime());
			stmt.setString(1, post.getTitulo());
			stmt.setString(2, post.getConteudo());
			stmt.setBoolean(3, post.isPublicado());
			stmt.setDate(4, data);
			stmt.setString(5, post.getBlogueiro().getLogin());
			stmt.execute();
		} catch (SQLException ex) {
			throw new RepositorioException(ex);
		} finally {
			finaliza();
		}
	}

	public void apagaPost(int id) throws RepositorioException {
		try {
			PreparedStatement stmt = preparaQuery("delete from post where id=?");
			stmt.setInt(1, id);
			stmt.execute();
		} catch (SQLException ex) {
			throw new RepositorioException(ex);
		} finally {
			finaliza();
		}
	}

	public void atualizaPost(Post post) throws RepositorioException {
		try {
			PreparedStatement stmt = preparaQuery("update post set titulo=?, conteudo=?, publicado=?, qtde_avaliacoes=?, media_avaliacoes=? where id=?");
			stmt.setString(1, post.getTitulo());
			stmt.setString(2, post.getConteudo());
			stmt.setBoolean(3, post.isPublicado());
			stmt.setInt(4, post.getAvaliacao().getQuantidadeNotas());
			stmt.setDouble(5, post.getAvaliacao().getMediaNotas());
			stmt.setInt(6, post.getId());
			stmt.execute();
		} catch (SQLException ex) {
			throw new RepositorioException(ex);
		} finally {
			finaliza();
		}
	}

	public List<Post> listaPostsPublicados() throws RepositorioException {
		String query = "select * from post inner join blogueiro on login = blogueiro_fk where publicado = 1 order by data desc";
		return listaPosts(query);
	}

	public List<Post> listaPosts() throws RepositorioException {
		String query = "select * from post inner join blogueiro on login = blogueiro_fk order by data desc";
		return listaPosts(query);
	}

	private List<Post> listaPosts(String query) throws RepositorioException {
		ResultSet rs = executaQuery(query);

		try {
			List<Post> lista = new ArrayList<Post>();
			while (rs.next()) {
				lista.add(popula(rs));
			}
			return lista;
		} catch (SQLException ex) {
			throw new RepositorioException(ex);
		} finally {
			finaliza();
		}
	}

	public Post pegaPost(int id) throws RepositorioException {
		try {
			PreparedStatement stmt = preparaQuery("select * from post inner join blogueiro on login = blogueiro_fk where id = ?");
			stmt.setInt(1, id);
			ResultSet rs = stmt.executeQuery();

			if (rs.next()) {
				return popula(rs);
			}

			throw new RepositorioException("Post nao encontrado");
		} catch (SQLException ex) {
			throw new RepositorioException(ex);
		} finally {
			finaliza();
		}
	}

	private Post popula(ResultSet rs) throws SQLException, RepositorioException {
		Calendar data = Calendar.getInstance();
		data.setTimeInMillis(rs.getDate("data").getTime());

		Post post = new Post();
		post.setConteudo(rs.getString("conteudo"));
		post.setData(data);
		post.setId(rs.getInt("id"));
		post.setTitulo(rs.getString("titulo"));
		post.setPublicado(rs.getBoolean("publicado"));

		Blogueiro blogueiro = new Blogueiro();
		blogueiro.setEmail(rs.getString("email"));
		blogueiro.setLogin(rs.getString("login"));
		blogueiro.setNome(rs.getString("nome"));
		post.setBlogueiro(blogueiro);

		Avaliacao avaliacao = new Avaliacao();
		avaliacao.setMediaNotas(rs.getDouble("media_avaliacoes"));
		avaliacao.setQuantidadeNotas(rs.getInt("qtde_avaliacoes"));
		post.setAvaliacao(avaliacao);

		populaComentarios(post);

		return post;
	}

	private void populaComentarios(Post post) throws SQLException,
			RepositorioException {
		PreparedStatement stmt = preparaQuery("select * from comentarios where post_fk = ? order by data desc");
		stmt.setInt(1, post.getId());
		ResultSet rs = stmt.executeQuery();

		List<Comentario> comentarios = new ArrayList<Comentario>();
		while (rs.next()) {
			Comentario comentario = new Comentario();
			comentario.setAutor(rs.getString("autor"));
			comentario.setConteudo(rs.getString("conteudo"));
			comentario.setData(rs.getDate("data"));
			comentario.setEmail(rs.getString("email"));

			comentarios.add(comentario);
		}

		post.setComentarios(comentarios);
	}

	public void adicionaComentario(Post post, Comentario comentario)
			throws RepositorioException {
		try {
			PreparedStatement stmt = preparaQuery("insert into comentarios (autor, email, data, conteudo,post_fk) values (?,?,?,?,?)");
			stmt.setString(1, comentario.getAutor());
			stmt.setString(2, comentario.getEmail());
			stmt.setDate(3, new Date(comentario.getData().getTime()));
			stmt.setString(4, comentario.getConteudo());
			stmt.setInt(5, post.getId());

			stmt.execute();
		} catch (SQLException ex) {
			throw new RepositorioException(ex);
		} finally {
			finaliza();
		}
	}
}
